<?php
namespace App\Http\Controllers\Api;

use App\Http\Controllers\BaseController;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Throwable;
use Exception;
use Carbon\Carbon;
use Illuminate\Validation\Rule;
use App\Services\User\IUserService;

class LoginController extends BaseController
{
    private $time;
    private $userService;

    public function __construct(IUserService $userService)
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
        $this->userService = $userService;
    }

    /**
     * 登录
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getLogin(Request $request)
    {
        $postData = $request->all();
        $validate = Validator::make($postData, [
            'name' => 'required|string',
            'pwd' => 'required|string'
        ]);
        if ($validate->fails()) return $this->getErrorMsg('1001', config('msg.common.1001'));
        try {
            $data = $this->userService->getLogin($postData);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }

    /**
     * 退出
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getLogout(Request $request)
    {
        $userToken = $request->header('token','');
        try {
            $data = $this->userService->getLogout($userToken);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }




    /**
     * 刷新token
     * @return array
     * @date 2021/07/29
     * @author live
     */
    public function getRefreshToken(Request $request)
    {
        $userToken = $request->header('token','');
        if ($userToken == '') return $this->getErrorMsg('1010', config('msg.common.1010'));
        try {
            $data = $this->userService->getRefreshToken($userToken);
            return $this->getSuccessMsg($data);
        } catch (Exception $e) {
            return $this->getErrorMsg($e->getCode(), $e->getMessage());
        }
    }

}