<?php
namespace App\Services\Script;

use App\Models\DepartmentManagerModel;
use App\Services\Script\IScriptService;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Cache;
use App\Facades\Util\Log;
use App\Models\WarnConfigModel;
use App\Models\LastIdModel;
use App\Models\WarnWaitModel;
use App\Models\UserCashModel;
use App\Models\SyceeWithdrawModel;
use App\Models\WarnPeopleModel;
use App\Models\WarnLogModel;

class ScriptService implements IScriptService
{
    private $time;

    public function __construct()
    {
        $this->time = Carbon::now()->format('Y-m-d H:i:s');
    }

    /**
     * 单次提现最高预警
     * @return int
     * @date 2021/08/02
     * @author live
     */
    public function getSingleWithdrawal()
    {
        $config = self::getWarnConfig(1);
        $oneMin = $config['oneMin'] ?? 0;
        if(empty($config) || $oneMin == 0) return true;
        $lastId = self::getLastDataId(2);

        $warnPeople = self::getWarnPeople();
        if(empty($warnPeople)) return true;
        $logs = SyceeWithdrawModel::where('sycee_withdraw.status',1)->where('sycee_withdraw.serial','>',$lastId)->where('sycee_withdraw.cash','>',$oneMin)->join('account','sycee_withdraw.user_id','account.user_id')->orderBy('sycee_withdraw.serial','ASC')->select('account.user_id','account.platform','account.platform_id','account.channel','sycee_withdraw.serial','sycee_withdraw.cash','sycee_withdraw.withdraw_time')->limit(100)->get();
        $array = [];
        $maxId = 0;
        foreach($logs as $log){
            $maxId = $log->serial;
            $key = $log->platform.'-'.$log->channel;
            $warnConfig = $config[$key] ?? [];
            if(empty($warnConfig)) continue;
            $people = $warnPeople[$key] ?? [];
            if(empty($people)) continue;
            if($log->cash < $warnConfig['one']) continue;

            $tem['ymd'] = date('Ymd',strtotime($log->withdraw_time));
            $tem['config_id'] = $warnConfig['id'];
            $tem['type'] = 1;
            $tem['cash'] = $log->cash;
            $tem['warn_value'] = $warnConfig['one'];
            $tem['people_id'] = implode('#',$people['id']);
            $tem['people_phone'] = implode('#',$people['phone']);
            $tem['people_name'] = implode('#',$people['name']);
            $tem['user_id'] = $log->platform_id;
            $tem['open_id'] = $log->user_id;
            $tem['status'] = 0;
            $tem['created_at'] = $this->time;
            $tem['updated_at'] = $this->time;
            $array[] = $tem;
        }

        Log::setGroup('Commands')->info('单次预警数据', ['info' => ['time'=>date('Y-m-d H:i:s'),'data'=>$array]]);
        DB::beginTransaction();
        try{
            if(!empty($array)) WarnWaitModel::insert($array);
            if($maxId > 0) self::getUpdateLastId(2,$maxId);
            unset($array,$tem,$config,$warnPeople,$logs,$log,$tem);
            DB::commit();
            return true;
        } catch (\Exception $e){
            DB::rollback();
            unset($array,$tem,$config,$warnPeople,$logs,$log,$tem);
            return false;
        }
    }


    /**
     * 获取预警人信息
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getWarnPeople()
    {
        $array = [];
        $peoples =  WarnPeopleModel::where('status',1)->select('id','platform','channel','name','phone')->get();
        foreach($peoples as $peo){
            $key = $peo->platform.'-'.$peo->channel;

            if(isset($array[$key])){
                $array[$key]['id'][] = $peo->id;
                $array[$key]['name'][] = $peo->name;
                $array[$key]['phone'][] = $peo->phone;
            }else{
                $tem['id'][] = $peo->id;
                $tem['name'][] = $peo->name;
                $tem['phone'][] = $peo->phone;
                $array[$key] = $tem;
            }
        }
        unset($peoples,$peo,$tem);
        return $array;
    }

    /**
     * 跑完脚本更新最后一组数据ID
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getUpdateLastId($type,$maxId)
    {
        $last = LastIdModel::where('type',$type)->select('id','last_id')->first();
        if($last){
            $last->last_id = $maxId;
        }else{
            $last = new LastIdModel();
            $last->type = $type;
            $last->last_id = $maxId;
            $last->created_at = Carbon::now()->format('Y-m-d H:i:s');
        }
        $last->save();
        return true;
    }

    /**
     * 获取最后一跑数据的ID
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getLastDataId($type)
    {
        $last = LastIdModel::where('type',$type)->select('id','last_id')->first();
        return $last->last_id ?? 0;
    }

    /**
     * 获取预警配置
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public static function getWarnConfig($type,$yf = [])
    {
        $array = ['oneMin'=>0,'dayMin'=>0,'userMin'=>0];
        $configs = WarnConfigModel::where('status',1);
        if(!empty($yf)) $configs->whereNotIn('id',$yf);
        $configs = $configs->select('id','platform','channel','one_withdrawal','day_withdrawal','people_day_withdrawal')->get();

        if(!$configs->count()) return [];
        foreach($configs as $con){
            $key = $con->platform.'-'.$con->channel;

            if($array['oneMin'] == 0 || $array['oneMin'] > $con->one_withdrawal) $array['oneMin'] = $con->one_withdrawal;
            if($array['dayMin'] == 0 || $array['dayMin'] > $con->day_withdrawal) $array['dayMin'] = $con->day_withdrawal;
            if($array['userMin'] == 0 || $array['userMin'] > $con->people_day_withdrawal) $array['userMin'] = $con->people_day_withdrawal;

            if(isset($array[$key])){
                if($array[$key]['one'] > $con->one_withdrawal){
                    $array[$key]['one'] = $con->one_withdrawal;
                    if($type == 1) $array[$key]['id'] = $con->id;
                }
                if($array[$key]['day'] > $con->day_withdrawal){
                    $array[$key]['day'] = $con->day_withdrawal;
                    if($type == 2) $array[$key]['id'] = $con->id;
                }
                if($array[$key]['user'] > $con->people_day_withdrawal){
                    $array[$key]['user'] = $con->people_day_withdrawal;
                    if($type == 3) $array[$key]['id'] = $con->id;
                }
            }else{
                $tem['id'] = $con->id;
                $tem['one'] = $con->one_withdrawal;
                $tem['day'] = $con->day_withdrawal;
                $tem['user'] = $con->people_day_withdrawal;
                $array[$key] = $tem;
            }
        }

        unset($configs,$con,$tem);
        return $array;
    }

    /**
     * 聚合玩家提现数据
     * @return array
     * @date 2021/08/02
     * @author live
     */
    public function getPolymerizeWithdrawal(){
        $lastId = self::getLastDataId(1);
        $logs = SyceeWithdrawModel::where('status',1)->where('sycee_withdraw.serial','>',$lastId)->join('account','sycee_withdraw.user_id','account.user_id')->orderBy('sycee_withdraw.serial','ASC')->select('account.user_id','account.platform','account.platform_id','account.channel','sycee_withdraw.serial','sycee_withdraw.cash','sycee_withdraw.withdraw_time')->limit(300)->get();
        $array = [];
        $maxId = 0;
        foreach($logs as $log){
            $maxId = $log->serial;
            $ymd = date('Ymd',strtotime($log->withdraw_time));
            $key = $log->platform_id.'-'.$ymd;
            if(isset($array[$key])){
                $array[$key]['cash'] = $array[$key]['cash'] + $log->cash;
            }else{
                $tem['ymd'] = $ymd;
                $tem['platform'] = $log->platform;
                $tem['channel'] = $log->channel;
                $tem['user_id'] = $log->platform_id;
                $tem['open_id'] = $log->user_id;
                $tem['cash'] = $log->cash;
                $tem['created_at'] = $this->time;
                $tem['updated_at'] = $this->time;
                $array[$key] = $tem;
            }
        }
        Log::setGroup('Commands')->info('聚合玩家提现数据', ['info' => ['time'=>date('Y-m-d H:i:s'),'data'=>$array]]);
        DB::beginTransaction();
        try{
            if(!empty($array)){
                foreach($array as $key=>$arr){
                    $verity = UserCashModel::where('ymd',$arr['ymd'])->where('user_id',$arr['user_id'])->select('id','cash')->first();
                    if($verity){
                        $verity->cash = $verity->cash + $arr['cash'];
                        $verity->updated_at = $this->time;
                        $verity->save();
                        unset($array[$key]);
                    }
                }

                if(!empty($array)) UserCashModel::insert($array);
            }
            if($maxId > 0) self::getUpdateLastId(1,$maxId);
            DB::commit();
            unset($logs,$log,$tem,$array);
            return true;
        } catch (\Exception $e){
            unset($logs,$log,$tem,$array);
            return false;
        }
    }

    /**
     * 清理历史数据
     * @return array
     * @date 2021/08/03
     * @author live
     */
    public function getClearLogData()
    {
        $day = date('Ymd',strtotime(date('Y-m-d')) - 432000);
        WarnWaitModel::where('ymd','<',$day)->delete();
        UserCashModel::where('ymd','<',$day)->delete();
        return true;
    }

    /**
     * 单日最高体现预警
     * @return array
     * @date 2021/08/03
     * @author live
     */
    public function getSingleDayWithdrawal()
    {
        $ymd = date('Ymd');
        $yfConfig = WarnWaitModel::where('ymd',$ymd)->where('type',2)->groupBy('config_id')->pluck('config_id');
        $config = self::getWarnConfig(2,$yfConfig->toArray());
        $dayMin = $config['dayMin'] ?? 0;
        if(empty($config) || $dayMin == 0) return true;
        $warnPeople = self::getWarnPeople();
        if(empty($warnPeople)) return true;

        $cash = UserCashModel::where('ymd',$ymd)->groupBy('platform','channel')->select('platform','channel',DB::raw('sum(cash) as cash'))->get();
        $array = [];
        foreach($cash as $log){
            $key = $log->platform.'-'.$log->channel;
            $warnConfig = $config[$key] ?? [];
            if(empty($warnConfig)) continue;
            $people = $warnPeople[$key] ?? [];
            if(empty($people)) continue;
            if($log->cash < $dayMin) continue;

            $tem['ymd'] = date('Ymd',strtotime($this->time));
            $tem['config_id'] = $warnConfig['id'];
            $tem['type'] = 2;
            $tem['cash'] = $log->cash;
            $tem['warn_value'] = $warnConfig['day'];
            $tem['people_id'] = implode('#',$people['id']);
            $tem['people_phone'] = implode('#',$people['phone']);
            $tem['people_name'] = implode('#',$people['name']);
            $tem['user_id'] = 0;
            $tem['open_id'] = '';
            $tem['status'] = 0;
            $tem['created_at'] = $this->time;
            $tem['updated_at'] = $this->time;
            $array[] = $tem;
        }
        Log::setGroup('Commands')->info('单日最高提现预警', ['info' => ['time'=>date('Y-m-d H:i:s'),'data'=>$array]]);
        if(!empty($array)){
            foreach($array as $key=>$arr){
                $verity = WarnWaitModel::where('ymd',$arr['ymd'])->where('config_id',$arr['config_id'])->where('type',2)->select('id')->first();
                if($verity) unset($array[$key]);
            }
            if(!empty($array)) WarnWaitModel::insert($array);
        }
        unset($logs,$log,$tem,$array);
        return true;
    }

    /**
     * 单日单用户最高体现预警
     * @return array
     * @date 2021/08/03
     * @author live
     */
    public function getSingleUserDayWithdrawal()
    {
        $ymd = date('Ymd');
        $config = self::getWarnConfig(3);
        $userMin = $config['userMin'] ?? 0;
        if(empty($config) || $userMin == 0) return true;
        $warnPeople = self::getWarnPeople();
        if(empty($warnPeople)) return true;
        $cash = UserCashModel::where('ymd',$ymd)->where('cash','>',$userMin)->select('platform','channel','user_id','open_id','cash')->get();
        $array = [];
        foreach($cash as $log){
            $key = $log->platform.'-'.$log->channel;
            $warnConfig = $config[$key] ?? [];
            if(empty($warnConfig)) continue;
            $people = $warnPeople[$key] ?? [];
            if(empty($people)) continue;

            $tem['ymd'] = date('Ymd',strtotime($this->time));
            $tem['config_id'] = $warnConfig['id'];
            $tem['type'] = 3;
            $tem['cash'] = $log->cash;
            $tem['warn_value'] = $warnConfig['user'];
            $tem['people_id'] = implode('#',$people['id']);
            $tem['people_phone'] = implode('#',$people['phone']);
            $tem['people_name'] = implode('#',$people['name']);
            $tem['user_id'] = $log->user_id;
            $tem['open_id'] = $log->open_id;
            $tem['status'] = 0;
            $tem['created_at'] = $this->time;
            $tem['updated_at'] = $this->time;
            $array[] = $tem;
        }
        Log::setGroup('Commands')->info('单日单用户最高提现预警', ['info' => ['time'=>date('Y-m-d H:i:s'),'data'=>$array]]);
        if(!empty($array)){
            foreach($array as $key=>$arr){
                $verity = WarnWaitModel::where('ymd',$arr['ymd'])->where('user_id',$arr['user_id'])->where('type',3)->select('id')->first();
                if($verity) unset($array[$key]);
            }
            if(!empty($array)) WarnWaitModel::insert($array);
        }
        unset($logs,$log,$tem,$array);
        return true;
    }

    /**
     * 发送钉钉消息
     * @return array
     * @date 2021/08/03
     * @author live
     */
    public function getSendDDMsg()
    {
        $sends = WarnWaitModel::where('status',0)->limit(100)->get();

        $logs = [];
        foreach($sends as $send){
            $message = '';
            $mobiles = str_replace('#',',',$send->people_phone);
            if($send->type == 1) $message = '【夜店红包版】单日单次提现:日期'.Carbon::parse($send->created_at)->format('Y-m-d H:i:s').',提现金额:'.sprintf('%.2f',$send->cash / 100);
            if($send->type == 2) $message = '【夜店红包版】单日累计提现:日期'.Carbon::parse($send->created_at)->format('Y-m-d H:i:s').',提现金额:'.sprintf('%.2f',$send->cash / 100);
            if($send->type == 3) $message = '【夜店红包版】单日单用户提现('.$send->user_id.'):日期'.Carbon::parse($send->created_at)->format('Y-m-d H:i:s').',提现金额:'.sprintf('%.2f',$send->cash / 100);
            $submitData = ['mobile_list'=>$mobiles,'message'=>$message,'api_token'=>config(env('ENVIRONMENT') . '.DINGDING_TOKEN')];
            postCurlJson(config(env('ENVIRONMENT') . '.DINGDING_URL'), $submitData, []);
            $send->status = 1;
            $send->save();

            $config = WarnConfigModel::where('id',$send->config_id)->select('id','platform','channel')->first();
            $tem['ymd'] = $send->ymd;
            $tem['platform'] = $config->platform ?? 0;
            $tem['channel'] = $config->channel ?? 0;
            $tem['cash'] = $send->cash;
            $tem['status'] = 1;
            $tem['type'] = $send->type;
            $tem['config_id'] = $send->config_id;
            $tem['people_id'] = $send->people_id;
            $tem['msg'] = $message;
            $tem['remark'] = '';
            $tem['send_time'] = $this->time;
            $tem['created_at'] = $this->time;
            $tem['updated_at'] = $this->time;
            $logs[] = $tem;

        }
        if(!empty($logs)) WarnLogModel::insert($logs);
        return true;
    }
}